"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reporter_1 = require("@hayspec/reporter");
class DefaultReporter {
    constructor(severities) {
        this.printer = new reporter_1.Printer();
        this.severities = severities || ['error', 'warning'];
    }
    onCompileStart(compiler) {
        this.printer.end('');
    }
    onCompileEnd(compiler) {
        if (compiler.output.errors) {
            this.onErrors(compiler);
        }
    }
    onSaveStart(compiler) {
        if (!compiler.output.errors) {
            this.onContracts(compiler);
        }
    }
    onSaveEnd(compiler) {
        this.onOverview(compiler);
        this.printer.end('');
    }
    onErrors(compiler) {
        this.severities.forEach((severity) => {
            const failures = compiler.output.errors
                .filter((e) => e.severity === severity);
            if (failures.length) {
                this.printer.end(this.printer.indent(1, ''), severity === 'error' ? 'Errors' : 'Warnings');
                failures.filter((e) => e.formattedMessage).forEach((failure) => {
                    const color = this.getErrorColor(failure);
                    this.printer.end(this.printer.indent(2, ''), this.printer.colorize(color, failure.formattedMessage.trim()));
                });
            }
        });
    }
    onContracts(compiler) {
        const contracts = Object.keys(compiler.output.contracts || {});
        if (contracts.length) {
            this.printer.end(this.printer.indent(1, ''), 'Contracts');
            contracts.forEach((contract) => {
                this.printer.end(this.printer.indent(2, ''), this.printer.colorize('gray', contract));
            });
        }
    }
    onOverview(compiler) {
        this.printer.end();
        const messages = [];
        const contracts = Object.keys(compiler.output.contracts || {}).length;
        if (contracts) {
            messages.push(this.printer.indent(1, ''), this.printer.colorize('greenBright', contracts), ' contracts');
        }
        if (this.severities.indexOf('error') !== -1) {
            const errors = (compiler.output.errors || []).filter((e) => e.severity === 'error').length;
            if (errors) {
                messages.push(this.printer.indent(1, ''), this.printer.colorize('redBright', errors), ' errors');
            }
        }
        if (this.severities.indexOf('warning') !== -1) {
            const warnings = (compiler.output.errors || []).filter((e) => e.severity === 'warning').length;
            if (warnings) {
                messages.push(this.printer.indent(1, ''), this.printer.colorize('yellowBright', warnings), ' warnings');
            }
        }
        if (messages.length) {
            this.printer.end(...messages);
        }
    }
    getErrorColor(error) {
        if (error.severity === 'error') {
            return 'redBright';
        }
        else {
            return 'yellowBright';
        }
    }
}
exports.DefaultReporter = DefaultReporter;
//# sourceMappingURL=reporter.js.map